@@ -304,7 +304,7 @@ class SwaggerMethod:
 
     def __init__(self, name: str, description: str, method: str, arguments: TYPE__flat_argument_response = None,
                  body: TYPE__argument_response = None, response: TYPE__responses = None,
-                 exceptions: TYPE__exceptions = None):
+                 exceptions: TYPE__exceptions = None, parallel_verification: bool = False):
         self._validate_exceptions(exceptions)
         self._validate_responses(response)
         validate_argument_responses(arguments)
@@ -313,6 +313,7 @@ class SwaggerMethod:
         self.name = name
         self.description = description
         self.method = method
+        self.parallel_verification = parallel_verification
 
         self.responses = response if isinstance_union(response, TYPE__swagger_list_like) else force_list(response)
         self.arguments = force_list(arguments)
